################################################################################
# File    : hsm2ast.tcl
# Content : ast generation used by ast2cpp.tcl
# History : 09-Feb-2004 - [rb] splitted hsm2cpp.tcl into parser.tcl,
#                              hsm2ast.tcl, ast2cpp.tcl & hsm2cpp.tcl
################################################################################

#   Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



# N.B. #########################################################################
# The functions here are using the globals : hsmDefinition & index


# tree generation ##############################################################

# parseHsm --
#
#   Parse a hsm
#

proc parseHsm {} {
	global hsmDefinition
	global index

	set name      [getName]
	set init      ""
	set enter     ""
	set leave     ""
	set timer     {}
	set actions   {}
	set subHsms   {}
	set outInputs {}
	set intInputs {}
	set outVars   {}
	set intVars   {}
	set initials  {}
	set virtuals  {}
	set locals    {}
	set hsmInputs {}
	set methods   {}
	set requis    {}

	set inputs {}
	set vars   {}

	set token [getToken {"\{" ";"}]

	# hsm declaration
	if {$token == ";"} {
		if {[lsearch [array names hsmDefinition] $name] < 0} {
			exitError "hsm '$name' isn't already defined."
		}
		return [lreplace $hsmDefinition($name) end end "declaration"]

	# state or machine definition
	} else {
		set nextTokens {"in" "\[" "require" "!" "var" "init" "enter" "leave" "virtual" "local" "-" "hsm" "\}"}
		while {1} {
			set token [getToken $nextTokens]
			switch -- $token {
				"in"      { set input [parseDecl]
				            set inputName [lindex $input 1]
				            if {[lsearch $hsmInputs $inputName] < 0} {
				            	lappend hsmInputs $inputName
				            }
				            if {[lsearch $inputs $inputName] < 0} {
				            	lappend inputs $inputName
				            	if {[lindex $input 2] == ""} {
				            		lappend outInputs $input
				            	} else {
				            		lappend intInputs $input
				            	}
				            } else {
				            	exitError "an input can't be redefined."
				            }
				          }
				"\["      { set condition [getBalanced "\[" "\]"]
				            getToken {":"}
								set nextHsm [getName]
								lappend initials [list $condition $nextHsm]
				          }
				"require" -
				"!"       { set condition [getParentized]
							getToken {"else" ":"}
							set code ""
							if {[getTokenExpected {"\{"}] == "\{"} {
								set code [getBalanced]
							}
#								if {[getToken {"|" ":"}] == "|"} {
#									getToken {"\{"}
#									set code [getBalanced]
#									getToken {":"}
#								}
				            lappend requis [linsert [linsert [parseEnterHsm] 0 $condition] end $code]
				          }
				"var"     { set var [parseDecl]
				            set varName [lindex $var 1]
				            if {[lsearch $vars $varName] < 0} {
				            	lappend vars $varName
				            	if {[lindex $var 2] == ""} {
				            		lappend outVars $var
				            	} else {
				            		lappend intVars $var
				            	}
				            } else {
				            	exitError "a variable can't be redefined."
				            }
				          }
				"init"  -
				"enter" -
				"leave"   { getToken {"\{"}
				            set $token [getBalanced]
				          }
				"virtual" -
				"local"   { lappend ${token}s [parseFunction] }
				"-"       { set action [parseAction]
				            set timer [lindex $action 7]
				            if {$timer == ""} {
							# non timer action
								if {[lindex $action 0] == "INTERNAL"} {
								# action triggered by a method
									set method [lindex $action 1]
									set args   [lindex $action 6]
									set action [lreplace $action 1 1 ""]
									if {[lsearch $methods $method] < 0} {
										lappend methods [list $method $args]
									}
								} else {
								# action triggered by an event
									set inputName [lindex $action 5]
									if {($inputName != "") & ([lsearch $hsmInputs $inputName] < 0)} {
										lappend hsmInputs $inputName
									}
									if {($inputName != "") & ([lsearch $inputs $inputName] < 0)} {
										lappend inputs $inputName
										set input {}
										lappend input ""
										lappend input $inputName
										lappend input ""
										lappend outInputs $input
									}
								}
				            }
				            lappend actions $action
				          }
				"hsm"     { set subHsm [parseHsm]
				            foreach input [lindex $subHsm $index(outInputs)] {
				            	set inputName [lindex $input 1]
				            	if {($inputName != "") & ([lsearch $inputs $inputName] < 0)} {
				            		lappend inputs $inputName
				            		lappend outInputs $input
				            	}
				            }
				            foreach var [lindex $subHsm $index(outVars)] {
				            	set varName [lindex $var 1]
				            	if {($varName != "") & ([lsearch $vars $varName] < 0)} {
				            		lappend vars $varName
				            		lappend outVars $var
				            	}
				            }
				            foreach method [lindex $subHsm $index(methods)] {
				            	if {[lsearch $methods $method] < 0} {
				            		lappend methods $method
				            	}
				            }
				            lappend subHsms $subHsm
				          }
				"\}"      { break }
				default   { exitError "unexpected '$token\'" }
			}
			# allow only single enter, leave, escape and after rule.
			if {[lsearch {"enter" "leave" "escape" "after"} $token] >= 0} {
				set nextTokens [lreplace $nextTokens [set i [lsearch $nextTokens "$token"]] $i]
			}
			# allow only in before subHsms or actions.
			# allow only var before subHsms.
			# it makes easier to compute inputs and vars list.
			if {$token == "hsm"} {
				set nextTokens [lreplace $nextTokens [set i [lsearch $nextTokens "in"]] $i]
				set nextTokens [lreplace $nextTokens [set i [lsearch $nextTokens "var"]] $i]
			}
			if {$token == "on"} {
				set nextTokens [lreplace $nextTokens [set i [lsearch $nextTokens "in"]] $i]
			}
		}
	}

	set actions [lsort -index 0 $actions]
#	set outInputs [lsort -index 1 $outInputs]
#	set outVars [lsort -index 1 $outVars]

	set res [list $name $init $enter $leave $timer $actions $subHsms $outInputs $intInputs $outVars $intVars $initials $virtuals $locals $hsmInputs $methods $requis "definition"]
	set hsmDefinition($name) $res

	return $res
}


# parseDecl --
#
#   Parse a declaration
#   <type> <name> [= <value>|(<value>)];
#
# return
#   {type name [value|__EMPTY__|""]}
#
#   - value if there is a value,
#   - __EMPTY__ if there is an empty
#		constructor like definition
#   - "" if there is no definition at all.

proc parseDecl {} {
	set decl ""
	set value ""

	# get declaration
	while {1} {
		set token [getWord]
		switch $token {
			";" -
			"=" -
			"("       { break }
			
			"\["      { exitError "array type are not supported" }

			"__EOF__" { exitError "variable initial value probably missing" }
		}
		append decl $token
	}

	set type [firstWords $decl]
	set name [lastWord $decl]

	# get initial value
	switch $token {
		"=" { while {1} {
		      	set token [getWord]
		      	if {$token == ";"} { break }
		      	if {$token == "__EOF__"} {
		      		exitError "EOF reached before parsing complete"
		      	}
		      	append value $token
		      }
		    }
		"(" { set value [getBalanced "(" ")"]
		      getToken {";"}
		    }
	}

	set type [string trim $type]
	set name [string trim $name]
	set value [string trim $value]
	if {($token == "(") && ($value == "")} {
		set value "__EMPTY__"
	}
	
	return [list $type $name $value]
}


# parseEnterHsm --
#
#   Parse a state to enter
#   <next>[(<var>=<value>,...)]
#
# return
#   {next {var value} ...}

proc parseEnterHsm {} {
	set name [getName]
	
	while {1} {
		if {([getTokenExpected {":"}] == ":") && ([getTokenExpected {":"}] == ":")} {
			append name "::[getName]"
		} else {
			break
		}
	}

	set res [list $name]

	# arguments to parse
	if {[getTokenExpected {"("}] == "("} {
		set token [getToken]
		if {$token != ")"} {
			while {1} {
				set var {}
				lappend var $token
				getToken {"="}

				set val [getParentized]
				lappend var $val

				lappend res $var

				set token [getToken {"," ")"}]
				if {$token == ")"} { break }
				set token [getToken]
			}
		}
	}

	return $res
}


# parseNextHsm --
#
#   Parse a next state rule
#   [[=][h]> <next>[(<var>=<value>,...)]]
#
# return
#   {transmit history next {var value} ...}

proc parseNextHsm {} {
	set res {}

	# a next state to parse
	set transmit [getTokenExpected {"="}]
	set history  [getTokenExpected {"h"}]
	
	if {[getTokenExpected {">"}] != ">"} {
	# no next state
		lappend res ""
	
	} else {
	# next state
		set res [linsert [parseEnterHsm] 0 $history]
		set res [linsert $res 0 $transmit]
	}
	
	return $res
}


# parseSend --
#
#   Parse a send rule
#   [send(<MSG>[,<value>])]
#
# return
#   the equivalent c++ code

proc parseSend {} {
	set code ""
	if {[getTokenExpected {"send" "dispatch" "broadcast"}] != "__UNEXPECTED__"} {
		getToken "("
		set msgToSend [getToken]
		if {[getTokenExpected ","] == ","} {
			lappend msgToSend [getBalanced "(" ")"]
		} else {
			getToken ")"
		}
		append code "\nhsm::Hsm *topHsm = top();\ntopHsm->broadcast(hsm::Event(topHsm, hsm::Event::[join $msgToSend ", "]));"
	}
	return $code
}


# parseAction --
#
#   Parse an action rule
#   - <sender>[.<MSG>]  |
#   - <method>(args)     > [[<cond>]] [{<code>}] [send(<MSG>[,<value>])] [> <next>]
#   - <MS>              |
#
# return
#   {evt sender cond code next input args ms}

proc parseAction {} {
	set evt    ""
	set sender ""
	set cond   {}
	set code   ""
	set input  ""
	set args   {}

	# parsing the message
	set ms [getParentized]
	
	if {[regexp -- {^([0-9]+)|(\(.*\))$} $ms]} {
	# time out transition (just a number or a parentized expression)
		set evt    "INTERNAL"
		set sender "this"
		lappend cond "(event.data.aInt == Interface::TimeOutEvent)"

	} else {
	# other transitions
		set evt $ms
		set ms ""
		
		if {[getTokenExpected "("] == "("} {
		# message trigered by a method
			if {[regexp -- {^[a-z]} $evt]} {} else {
				exitError "method name expected \na method name must begin with a lower case letter."
			}
			if {$evt == "timeOut"} {
				exitError "an event invocation method can't be named 'timeOut'."
			}
			set method [string toupper $evt 0 0]
			set sender $evt
			set evt    "INTERNAL"
			lappend cond   "(event.data.aInt == Interface::${method}Event)"
			foreach arg [split [getBalanced "(" ")"] ","] {
				lappend args [list [firstWords $arg] [lastWord $arg]]
			}
			if {$args != {}} {
				set code "Interface::${method}Args *args = reinterpret_cast< Interface::${method}Args * >(event.args);\n"
				foreach arg $args {
					set type [lindex $arg 0]
					set name [lindex $arg 1]
					append code "$type $name = args->$name;\n"
				}
				append code "delete args;\n\n"
			}
		
		} else {
		# message trigered by an event
			if {[getTokenExpected "."] == "."} {
			# sender.EVT
				set sender $evt
				set evt [getToken]
			
			} else {
			# sender or EVT
				if {[regexp -- {^[A-Z]+$} $evt]} {} else {
					set sender $evt
					set evt "CHANGE"
				}
			}
			
			# if sender is not parentized, add it as input
			if {[string equal -length 1 $sender "("] == 0} {
				set input $sender
			}
		}
	}
	
	# parsing the rest of the rule
	while {1} {
		set token [getTokenExpected {"\[" "\{"}]
		switch $token {
			"\[" { lappend cond "([getBalanced "\[" "\]"])" }
			"\{" { append code [getBalanced] }
			"__UNEXPECTED__" { break }
		}
	}
	set cond [join $cond " && "]

	append code [parseSend]
	set next [parseNextHsm]
	
	return [list $evt $sender $cond $code $next $input $args $ms]
}


# parseFunction --
#
#   Parse a function rule
#   virtual|local <type> <name>(<args>) <qual> {<code>}
#
# return
#   {"type name(args) qual" code}

proc parseFunction {} {
	set decl [getBalanced ")" "("]
	set type [firstWords $decl]
	set name [lastWord $decl]
	set args [getBalanced "(" ")"]
	set qual [getBalanced "\}" "\{"]
	set code [getBalanced]


	return [list "$type ${name}($args) $qual" $code]
}
