#include <w3c/svg/SVGElement.hpp>
#include <w3c/svg/SVGRectElement.hpp>
#include <w3c/svg/SVGCircleElement.hpp>

#include <hsm/svgl/SVGLWindow.h>
#include <hsm/svgl/SVGLWindowPicking.h>
#include <hsm/interaction/Protocol.h>
#include <hsm/svgl/SVGLBehaviour.h>
#include <hsm/svgl/Svgl.h>

#include <hsm/util/Debug.h>

#include <Protocol.h>


/* CreateRectangle ************************************************************/

class CreateRectangle : public hsm::SVGLBehaviour, public hsm::Translate {
	static const hsm::SVGLBehaviour::Factory::Register< CreateRectangle > registration;

	svg::SVGGElement *palette;
	hsm::Translate   *handle;
	
public :
	CreateRectangle(svg::SVGElement *e, hsm::SVGLWindow *w) : hsm::SVGLBehaviour(e, w), handle(0) {
		palette = dynamic_cast< svg::SVGGElement * >(elem->getParentNode()->getParentNode());
	}
	
	virtual bool begin() {
		window->lock();
		
		window->svgFeedback->removeChild(palette);
		
		svg::SVGRectElement *rect = new svg::SVGRectElement(window->svgDocument);
		rect->setX(0 + pick/hsm::Point::X);
		rect->setY(0 + pick/hsm::Point::Y);
		rect->setWidth(0);
		rect->setHeight(0);
		rect->setStroke("black");
		rect->setFill("white");
		hsm::svgl::setAttribute(rect, "hsm:behaviour", "rectangle");
		window->svgContent->appendChild(rect);
		window->instanciateBehaviour(rect);
		
		window->unlock();

		hsm::Point sp(2);
		window->window2screen(pick, sp);
		handle = window->pick< hsm::Translate >(sp);
		if(handle != 0) return handle->begin();
		return false;
	}
	
	virtual void end() {
		if(handle != 0) handle->end();
		window->lock();
		window->svgFeedback->appendChild(palette);
		window->unlock();
	}
	
	virtual void translate(const hsm::Point &p) {
		if(handle != 0) handle->translate(p);
	}
};

const hsm::SVGLBehaviour::Factory::Register< CreateRectangle > CreateRectangle::registration("createRectangle");


/* CreateCircle ***************************************************************/

class CreateCircle : public hsm::SVGLBehaviour, public hsm::Translate {
	static const hsm::SVGLBehaviour::Factory::Register< CreateCircle > registration;

	svg::SVGGElement *palette;
	hsm::Translate   *handle;
	
public :
	CreateCircle(svg::SVGElement *e, hsm::SVGLWindow *w) : hsm::SVGLBehaviour(e, w), handle(0) {
		palette = dynamic_cast< svg::SVGGElement * >(elem->getParentNode()->getParentNode());
	}
	
	virtual bool begin() {
		window->lock();
		
		window->svgFeedback->removeChild(palette);
		
		svg::SVGCircleElement *circle = new svg::SVGCircleElement(window->svgDocument);
		circle->setCx(0 + pick/hsm::Point::X);
		circle->setCy(0 + pick/hsm::Point::Y);
		circle->setR(0);
		circle->setStroke("black");
		circle->setFill("white");
		hsm::svgl::setAttribute(circle, "hsm:behaviour", "circle");
		window->svgContent->appendChild(circle);
		window->instanciateBehaviour(circle);
		
		window->unlock();

		hsm::Point sp(2);
		window->window2screen(pick, sp);
		handle = window->pick< hsm::Translate >(sp);
		if(handle != 0) return handle->begin();
		return false;
	}
	
	virtual void end() {
		if(handle != 0) handle->end();
		window->lock();
		window->svgFeedback->appendChild(palette);
		window->unlock();
	}
	
	virtual void translate(const hsm::Point &p) {
		if(handle != 0) handle->translate(p);
	}
};

const hsm::SVGLBehaviour::Factory::Register< CreateCircle > CreateCircle::registration("createCircle");


/* Delete *********************************************************************/

class Delete : public hsm::SVGLBehaviour, public hsm::PushPop, public hsm::Translate {
	static const hsm::SVGLBehaviour::Factory::Register< Delete > registration;

public :
	Delete(svg::SVGElement *e, hsm::SVGLWindow *w) : hsm::SVGLBehaviour(e, w) {}

	virtual void push() {
		hsm::Point sp(2);
		window->window2screen(pick, sp);
		Deletable *toDelete = window->pick< Deletable >(sp, 0, false);
		if(toDelete != 0) {
			toDelete->remove();
		}
	}
};

const hsm::SVGLBehaviour::Factory::Register< Delete > Delete::registration("delete");


/* Raise / Lower **************************************************************/

class Raise : public hsm::SVGLBehaviour, public hsm::PushPop, public hsm::Translate {
	static const hsm::SVGLBehaviour::Factory::Register< Raise > registration;

	svg::SVGGElement *palette;
	
public :
	Raise(svg::SVGElement *e, hsm::SVGLWindow *w) : hsm::SVGLBehaviour(e, w) {
		palette = dynamic_cast< svg::SVGGElement * >(elem->getParentNode()->getParentNode());
	}

	virtual void push() {
		hsm::Point sp(2);
		window->window2screen(pick, sp);
		RaiseLower *toRaise = window->pick< RaiseLower >(sp, 0, false);
		if(toRaise != 0) {
			if(toRaise->begin()) {
				toRaise->raise();
			}
			toRaise->end();
		}
	}
};

const hsm::SVGLBehaviour::Factory::Register< Raise > Raise::registration("raise");


class Lower : public hsm::SVGLBehaviour, public hsm::PushPop, public hsm::Translate {
	static const hsm::SVGLBehaviour::Factory::Register< Lower > registration;

public :
	Lower(svg::SVGElement *e, hsm::SVGLWindow *w) : hsm::SVGLBehaviour(e, w) {}
	
	virtual void push() {
		hsm::Point sp(2);
		window->window2screen(pick, sp);
		RaiseLower *toLower = window->pick< RaiseLower >(sp, 0, false);
		if(toLower != 0) {
			if(toLower->begin()) {
				toLower->lower();
			}
			toLower->end();
		}
	}
};

const hsm::SVGLBehaviour::Factory::Register< Lower > Lower::registration("lower");


/* ColorSetter ****************************************************************/

class ColorSetter : public hsm::SVGLBehaviour, public hsm::PushPop, public hsm::Translate {
	static const hsm::SVGLBehaviour::Factory::Register< ColorSetter > registration;

	Colorable *toColor;
	hsm::Point position;
	
public :
	ColorSetter(svg::SVGElement *e, hsm::SVGLWindow *w) : hsm::SVGLBehaviour(e, w), position(1) {}
	
	void setColor() {
		float sx, sy, tx, ty;
		hsm::svgl::getGlobalTransform(elem, sx, sy, tx, ty, false);
		float X = (position/hsm::Point::X - tx) / 18.;
		float r, g, b;
		if(X < .5) {
			r = g = 256. * X * 2.;
			b = 256. - r;
		} else {
			r = 256.; b = 0;
			g = 256. * (1 - X) * 2.;
		}
		if(r > 256) r = 256; if(r < 0) r = 0;
		if(g > 256) g = 256; if(g < 0) g = 0;
		if(b > 256) b = 256; if(b < 0) b = 0;
		
		if(toColor != 0) toColor->setColor(r, g, b);	
	}

	virtual void push() {
		hsm::Point sp(2);
		window->window2screen(pick, sp);
		toColor = window->pick< Colorable >(sp, 0, false);
		if(toColor != 0) toColor->begin();
		position = pick;
		setColor();
	}
	
	virtual void pop() {
		if(toColor != 0) toColor->end();
	}
	
	virtual void translate(const hsm::Point &p) { 
		position += p;
		setColor();
	}
};

const hsm::SVGLBehaviour::Factory::Register< ColorSetter > ColorSetter::registration("color");
