////////////////////////////////////////////////////////////////////////////////
// File    : svgViewer.cpp
// Content : a svg viewer
// History : 11-Jun-2003 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/* includes *******************************************************************/

#include <cstdlib>
#include <iostream>
#include <sstream>
using namespace std;

#include <svgl/Parser.hpp>
#include <svgl/InitHelper.hpp>

#include <w3c/svg/SVGSVGElement.hpp>
	
#include <hsm/svgl/SVGLWindow.h>
#include <hsm/system/window/EventHandler.h>
#include <hsm/system/event/EventLoop.h>
#include <hsm/component/Point.h>
#include <hsm/component/ButtonPad.h>
#include <hsm/system/plugin/PlugIn.h>

#include <interactions.h>

#include <hsm/svgl/Svgl.h>


/* handle quit ****************************************************************/

void quitHandler() {
	cout << endl << "bye." << endl;
}


/* main ***********************************************************************/

int main(int argc, const char *argv[]) {
	cout << "svgViewer - svg viewer using svgl & hsm." << endl << flush;
	atexit(quitHandler);
	
	if(argc != 2) {
		cerr << "usage : svgViewer file.svg" << endl;
		return EXIT_FAILURE;
	}
	
	// setting up the window and the pointer
	hsm::SVGLWindow *window = new hsm::impl::Component< hsm::SVGLWindow, hsm::processing::MonoAsync >("svgViewer");
	hsm::EventHandler *pointer = hsm::repository::getEventHandler("Pointer");
	window->addEventHandler(pointer);
	hsm::EventHandler *keyboard = hsm::repository::getEventHandler("Keyboard");
	window->addEventHandler(keyboard);

	// creating instruments
	Translater::Hsm *translater = new Translater::Hsm(Translater::Data(window, *pointer/"buttons"/hsm::ButtonPad::L, *pointer/"position"));
	Zoomer::Hsm *zoomer = new Zoomer::Hsm(Zoomer::Data(window, *pointer/"position"/hsm::Point::cast, *pointer/"wheel"/hsm::Point::W));
	EnterLeaver::Hsm *enterLeaver = new EnterLeaver::Hsm(EnterLeaver::Data(window, *pointer/"position"));
	
	TabletManager::Hsm *manager = new TabletManager::Hsm(TabletManager::Data(window));
	
	KZoomer::Hsm *kzoomer = new KZoomer::Hsm(KZoomer::Data(window, *pointer/"position"/hsm::Point::cast, *keyboard/"keys"));


	// getting the svg document
	svgl::Parser parser;
	svg::SVGDocument *document = 0;
	document = parser.parse(argv[1]);
	if(document == 0) {
		std::stringstream strstr;
		strstr << "<?xml version='1.0' encoding='iso-8859-1'?>"
		          "<!DOCTYPE svg PUBLIC '-//W3C//DTD SVG 03December 1999//EN' 'http://www.w3.org/Graphics/SVG/SVG-19991203.dtd'>"
		          "<svg>  <text x='10' y='40' style='font-size:30; fill:black'> wrong file : " << argv[1] << "</text> </svg>";
		document = parser.parseFromStream(&strstr);
	}

	window->setDocument(document);

	hsm::svgl::setAttribute(window->svgContent, "hsm:behaviour", "visual/PanAndZoomable");
	window->initBehaviours();
	
	// launching interaction
	window->show(true);
	hsm::eventloop::loop();	


	// clean-up
	delete translater;
	delete zoomer;
	delete enterLeaver;
	
	delete manager;
	
	delete kzoomer;
	
	delete window;
	

	return EXIT_SUCCESS;
}