################################################################################
# File    : config.mk
# Author  : Renaud Blanch
# Content : OS, compiler & libraries specific flags
# History : 15-Nov-2001 - [rb] creation Win32 version
#           25-Jan-2002 - [rb] updated to MacOSX version
#           31-Jan-2002 - [rb] updated to make the distinction between shared
#                         library and loadable objects (modules or bundle in
#                         MacOSX terminology)
#           28-Mar-2002 - [rb] added openGL support
#           25-Jun-2002 - [rb] started Linux version
#           09-Jun-2004 - [rb] added svgl support
################################################################################

#   Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


# default compiler is gcc
COMPILER := GCC

ifeq ($(PLATFORM), X)
SUF_OBJ := .o
SUF_EXE :=
PRE_SL := lib
SUF_SL := .so
SUF_LO := .so
endif

ifeq ($(PLATFORM), MACOSX)
SUF_OBJ := .o
SUF_EXE :=
PRE_SL := lib
SUF_SL := .dylib
SUF_LO := .bundle
CXXFLAGS := -fno-coalesce
GLLIBS := -framework OpenGL -framework AGL

endif

ifeq ($(PLATFORM), WIN32)
COMPILER := MSVC6
SUF_OBJ := .obj
SUF_EXE := .exe
PRE_SL :=
SUF_SL := .dll
SUF_LO := .dll
LIBS := user32.lib
GLLIBS := opengl32.lib 
ifeq ($(WINTAB), YES)
CPPFLAGS += -DHAVE_WINTAB
INCDIRS += -I"$(WTDIR)/Include"
WTLIB := $(WTDIR)/LIB/I386/wintab32.lib
endif
endif


ifeq ($(COMPILER), GCC)
CXX := g++
ifeq ($(DISTRIBUTED),)
DCXX := g++
else
DCXX := distcc
endif
CXXFLAGS += -Wall -ansi -pedantic
ifeq ($(VERSION), DEBUG)
CXXFLAGS += -O0 -g
else
CXXFLAGS += -O3
endif
endif

ifeq ($(COMPILER), MSVC6)
CXX := cl -nologo
DCXX := cl -nologo
CXXFLAGS := -GX -GR -MD -vmg
ifeq ($(VERSION), DEBUG)
CXXFLAGS += -Od -Zi
else
CXXFLAGS += -Ox
endif
endif

SUF_PL := .$(PLATFORM)

CPPFLAGS += -D$(PLATFORM) -D$(VERSION)_VERSION


# optional features

ifeq ($(SVGL), DYLIB)
SVGL_LDFLAGS := $(shell pkg-config svgl --libs-only-L)
SVGL_LIBS := $(shell pkg-config svgl --libs-only-l)
SVGL_LIBS += $(shell pkg-config svgl --libs-only-other)
SVGL_CPPFLAGS := $(shell pkg-config svgl --cflags)
endif

ifeq ($(SVGL), FRAMEWORK)
SVGL_LIBS := "-framework svgl"
SVGL_CPPFLAGS := "-framework svgl"
endif