////////////////////////////////////////////////////////////////////////////////
// File    : Composite.h
// Content : a common interface class for hsm composite component
// History : 10-Jan-2004 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/// \file Composite.h
/// libhsm composite components definition.


#if !defined(HSM_COMPOSITE_H)
#define HSM_COMPOSITE_H


/* includes *******************************************************************/

#include <hsm/core/Component.h>


namespace hsm {

/* Composite class declaration ************************************************/

	/// composite component class definition
	class Composite : public Component {
	public :
		/// \c cast is used as type specifier for dynamic_casting operator
		/// (see Component.h)
		static const Composite *cast;
				
	protected :
		typedef std::map< Id, Component * > Map;
		Map components;

	public :
		/// constructor
		/// \param id id of the component
		         Composite(const Id &id = Id());
		virtual ~Composite(); ///< destructor

		/// adds a component as children of the composite
		/// \param component the component to add
		void add(Component *component);
		
		/// removes a children component of the composite
		/// \param component the component to remove
		void remove(Component *component);


		/// gets a children
		/// \param id the name of the children
		/// \return the component
		Component *get(const Id &id) const;

		friend Component &operator/(Component &component, const Id &id);
		friend Component &operator/(Component &component, const char *id);
	};

	
} // namespace hsm


#endif // !defined(HSM_COMPOSITE_H)