////////////////////////////////////////////////////////////////////////////////
// File    : Svgl.h
// Content : svgl helper functions
// History : 30-Jan-2004 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#if !defined(HSM_SVGL_H)
#define HSM_SVGL_H


/* includes *******************************************************************/

#include <sstream>

#include <w3c/svg/Element.hpp>


namespace hsm {

	namespace svgl {
	
/* local transformation computation *******************************************/

		void getGlobalTransform(svg::Element *e, float &sx, float &sy, float &tx, float &ty, bool skipFirst = true);
//		void getGlobalTransform(svg::Element *e, float &sx, float &sy, float &tx, float &ty, float &angle, bool skipFirst = true);
		
		
/* svg attribute getters and setters ******************************************/

		std::string getAttributeCC(svg::Element *e, const char *attribute);
		void setAttributeCC(svg::Element *e, const char *attribute, const char *value);
		
		template< class T >
		void getAttribute(svg::Element *e, const char *attribute, T &value) {
			std::stringstream ss;
			ss << getAttributeCC(e, attribute);
			ss >> value;
		}

		template< class T >
		void getAttribute(svg::Element *e, const char *attribute, T *&value) {
			int p;
			std::stringstream ss;
			ss << getAttributeCC(e, attribute);
			ss >> p;
			value = reinterpret_cast< T * >(p);
		}
		
		template< class T >
		void setAttribute(svg::Element *e, const char *attribute, T const &value) {
			std::stringstream ss;
			ss << value;
			setAttributeCC(e, attribute, ss.str().c_str());
		}
		
		template< class T >
		void setAttribute(svg::Element *e, const char *attribute, T * const &value) {
			std::stringstream ss;
			ss << reinterpret_cast< int >(value);
			setAttributeCC(e, attribute, ss.str().c_str());
		}
		

/* typed svg tree walking *****************************************************/
		
		template< class C >
		C getSibling(dom::Node *&child) {
			C result = 0;
			while((result == 0) && (child != 0)) {
				result = dynamic_cast< C >(child);
				child = child->getNextSibling();
			}
			return result;
		}


	} // namespace svgl
	
} // namespace hsm


#endif // !defined(HSM_SVGL_H)
