////////////////////////////////////////////////////////////////////////////////
// File    : EventLoop.cpp
// Content : functions dealing with system events
// History : 07-Jan-2004 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/



/* includes *******************************************************************/

#include <EventLoop.h>

#if defined(MACOSX)
#  include <Carbon/Carbon.h>
#elif defined(WIN32)
#  define _WIN32_WINNT 0x0400
#  include <windows.h>
#endif


namespace hsm {

	namespace eventloop {
		
/* functions ******************************************************************/

		bool step() {
			bool res = true;

#if defined(MACOSX) ////////////////////////////////////////////////////////////
			EventRef event;
			ReceiveNextEvent(0, 0, kEventDurationForever, true, &event);

			if((GetEventClass(event) == kEventClassKeyboard) && (GetEventKind(event) == kEventRawKeyDown)) {
				unsigned char keyMacCharCodes;
				GetEventParameter(event, kEventParamKeyMacCharCodes, typeChar, 0, sizeof(char), 0, &keyMacCharCodes);
//				UInt32 keyCode;
//				GetEventParameter(event, kEventParamKeyCode, typeUInt32, 0, sizeof(UInt32), 0, &keyCode);
//				if(keyCode == 53) { // 53 is keycode for esc.
				if(keyMacCharCodes == kEscapeCharCode) {
					res = false;
				}
			}
			SendEventToEventTarget(event, GetEventDispatcherTarget());
			ReleaseEvent(event);

#elif defined(WIN32) ///////////////////////////////////////////////////////////
			MSG msg;
			GetMessage(&msg, 0, 0, 0);

			if((msg.message == WM_KEYDOWN) && (msg.wParam == VK_ESCAPE)) {
				res = false;
			}
			TranslateMessage(&msg);
			DispatchMessage(&msg);

#endif /////////////////////////////////////////////////////////////////////////
			return res;
		}

		void loop() {
			while(step()) {}
		}


	} // namespace eventloop

} // namespace hsm
