////////////////////////////////////////////////////////////////////////////////
// File    : File.h
// Content : file relative platform dependent declaration
// History : 09-Jan-2004 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/// \file File.h
/// Os level file related definitions


#if !defined(HSM_FILE_H)
#define HSM_FILE_H


/* includes *******************************************************************/

#if defined(WIN32)
#define _WIN32_WINNT 0x0400
#include <windows.h>
#endif


namespace hsm {

	/// contains some file related definitions
	namespace file {

/* typedefs *******************************************************************/

	#if !defined(WIN32)
		typedef int    Descriptor; ///< file descriptor
	#else
		typedef HANDLE Descriptor; ///< file descriptor
	#endif


/* functions ******************************************************************/

		/// returns the number of bytes available for reading on a given file descriptor
		/// \return the number of bytes available
		/// \param id the file descriptor
		int bytesToRead(Descriptor id);


	} // namespace file

} // namespace hsm


#if defined(WIN32)

namespace hsm {

	namespace file {

		int close(Descriptor fildes);
		int pipe(Descriptor *fildes);
		int read(Descriptor fildes, void *buf, int nbyte);
		int write(Descriptor fildes, void *buf, int nbyte);

	} // namespace file

} // namespace hsm

using hsm::file::close;
using hsm::file::pipe;
using hsm::file::read;
using hsm::file::write;

#endif


#endif // !defined(HSM_FILE_H)