////////////////////////////////////////////////////////////////////////////////
// File    : Selectable.h
// Content : base class for selectable things
// History : 13-Feb-2003 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/// \file Selectable.h
/// a class to encapsulate selectable things


#if !defined(HSM_SELECTABLE_H)
#define HSM_SELECTABLE_H


/* includes *******************************************************************/

#include <hsm/system/select/File.h>

#include <map>


namespace hsm {

/* Selectable class declaration ************************************************/

	/// implements a selectable object
	/// \see hsm::Selector
	class Selectable {
	protected :
		typedef std::map< file::Descriptor, Selectable * > Map;
		static Map selectableMap;

	protected :
		file::Descriptor fd;

	public :
		/// constructor
		/// \param id the real os descriptor of the file
		Selectable(file::Descriptor id = 0);
		
		/// destructor
		virtual ~Selectable();
		
		
		/// casts the Selectable object to file::Descriptor
		/// \return the os descriptor of the object
		operator file::Descriptor() const;
		
		/// gets a Selectable given a os descriptor
		/// \param id the os descriptor of the file
		/// \return a pointer on the corresponding Selectable
		static Selectable *get(const file::Descriptor &id);

	protected :
		void registerFd();
	};


} // namespace hsm


#endif // !defined(HSM_SELECTABLE_H)