////////////////////////////////////////////////////////////////////////////////
// File    : Selector.WIN32.cpp
// Content : A manager for select operations
// History : 31-Jan-2003 - [rb] creation
//           08-Feb-2003 - [rb] rewrite, added add / remove
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/* includes *******************************************************************/

#include <Selector.h>

#include <hsm/system/select/Alert.h>
#include <hsm/system/thread/Mutex.h>
#include <hsm/core/Event.h>
#include <hsm/core/Component.h>
#include <hsm/util/Debug.h>


namespace hsm {

/* Selector implementation ****************************************************/

	void Selector::os::watch(Selector *s) {
		s->statusMutex->lock();
		DWORD count = s->statusMapMap.size() + 1;
		file::Descriptor *fileDescs = new file::Descriptor[count];

		fileDescs[0] = *(s->registrationAlert);

		unsigned int index = 1;
		for(StatusMapMap::const_iterator smmi = s->statusMapMap.begin(); smmi != s->statusMapMap.end(); ++smmi) {
			file::Descriptor fd = (*smmi).first;
			fileDescs[index] = fd;
			++index;
		}
		s->statusMutex->unlock();

		DWORD delay = (s->timeOut == 0 ? INFINITE : s->timeOut);
		DWORD signaledIndex = WaitForMultipleObjects(count, fileDescs, false, delay) - WAIT_OBJECT_0;

		if(signaledIndex == 0) {
			s->registrationAlert->reset();
		}

		s->statusMutex->lock();
		for(index = signaledIndex; index < count; index++) {
			file::Descriptor fd(fileDescs[index]);
			Status stat = RWE;
			if((index == signaledIndex) || (WaitForSingleObject(fd, 0) != WAIT_TIMEOUT)){
				StatusMap &sm = s->statusMapMap[fd];
				for(StatusMap::const_iterator smi = sm.begin(); smi != sm.end(); ++smi) {
					Functor *f = (*smi).first;
					Status status = (*smi).second;
					if(f != 0) {
						if(stat & status) (*f)(new Event(0, stat, fd));
					}
				}
			}
		}
		s->statusMutex->unlock();

		delete[] fileDescs;
	}


} // namespace hsm