////////////////////////////////////////////////////////////////////////////////
// File    : Template.h
// Content : macros to check template properties at compile time
//           original ideas taken from Aleandrescu's Modern C++ Design book :
//           [http://www.moderncppdesign.com],
//           and from boost.org [http://boost.org].
// History : 08-Feb-2004 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/// \file Template.h
/// Usefull template argument properties static checking.


#if !defined(HSM_TEMPLATE_H)
#define HSM_TEMPLATE_H


namespace hsm {

/* compile time assertion *****************************************************/

	template< int > struct Check {};

	#define COMPILE_ASSERT(expr, msg) struct CHECK_##msg { enum { TEST_##msg = (expr) ? 1 : -1 }; char COMPILE_ASSERT_FAILED_##msg[TEST_##msg]; }; \
	                                  typedef hsm::Check< sizeof(CHECK_##msg) > DO_CHECK_##msg;


/* convertibility detection ***************************************************/

	template< class T, class U >
	struct Conversion {
		class Big { char dummy[2]; };
		static char test(U);
		static Big  test(...);
		static T    makeT();
		static const bool result;
	};

	#define CONVERSION(T, U) hsm::Conversion< T, U >::result

	template< class T, class U >
	const bool CONVERSION(T, U) = (sizeof(Conversion< T, U >::test(Conversion< T, U >::makeT())) == sizeof(char));


/* same type detection ********************************************************/

	template< class T, class U >
	struct SameType { static const bool result; };
	template< class T >
	struct SameType< T, T > { static const bool result; };

	#define SAME_TYPE(T, U) hsm::SameType< T, U >::result

	template< class T, class U >
	const bool SAME_TYPE(T, U) = 0;
	template< class T >
	const bool SAME_TYPE(T, T) = 1;


/* inheritance detection ******************************************************/

	template< class T, class U >
	struct BaseOf { static const bool result; };

	#define BASE_OF(T, U) hsm::BaseOf< T, U >::result

	template< class T, class U >
	const bool BASE_OF(T, U) = CONVERSION(const U *, const T *) && !SAME_TYPE(const T *, const void *);


} // namespace hsm


#endif // !defined(HSM_TEMPLATE_H)