////////////////////////////////////////////////////////////////////////////////
// File    : test-SVGLWindow.cpp
// Content : a test for hsmSVGLWindow
// History : 09-Dec-2002 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2003  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/* includes *******************************************************************/

#include <cstdlib>
#include <iostream>
using namespace std;

#include <hsm/svgl/SVGLWindow.h>
#include <hsm/component/Point.h>
#include <hsm/system/window/EventHandler.h>
#include <hsm/system/event/EventLoop.h>
#include <hsm/core/Component.h>

#include <hsm/system/thread/ThreadLoop.h>
#include <hsm/util/Functor.h>

#include <w3c/svg/SVGRectElement.hpp>


/* globals ********************************************************************/

class Translator : public hsm::Functor {
protected :
	hsm::SVGLWindow *window;
	svg::SVGRectElement *rect;

	float x, y;
	float dx, dy;
	
public :
	Translator(hsm::SVGLWindow *w) : window(w), x(0), y(0), dx(0.2), dy(0.3) {
		rect = new svg::SVGRectElement(window->svgDocument);
		
		rect->setWidth(40);
		rect->setHeight(30);
		rect->setX(-20);
		rect->setY(-15);
		
		rect->setFill(0, 0, 1);
		rect->setStroke(0, 0, 0);
		rect->setStrokeWidth(2);
		
		window->svgContent->appendChild(rect);		
	}
	
	virtual void operator()(void *pData = 0) { 
		x += dx; y += dy;
		window->lock();
		rect->translate(x, y);
		window->unlock();
		if(x > *window/"size"/hsm::Point::X) { x = *window/"size"/hsm::Point::X; dx = -dx; }
		if(y > *window/"size"/hsm::Point::Y) { y = *window/"size"/hsm::Point::Y; dy = -dy; }
		if(x < 0) { x = 0; dx = -dx; }
		if(y < 0) { y = 0; dy = -dy; }
	}
};


/* handle quit ****************************************************************/

void quitHandler() {
	cout << endl << "bye." << endl;
}


/* main ***********************************************************************/

int main(int argc, const char *argv[]) {
	cout << "test-SVGLWindow - a program to test the hsm SVGLWindow class." << endl << endl;
	atexit(quitHandler);

	hsm::SVGLWindow *window = new hsm::impl::Component< hsm::SVGLWindow >("SVGLWindow test");
	
	hsm::Component *listener = hsm::repository::getComponent("Listener");
	(*window/"position").addReceiver(listener);
	(*window/"size").addReceiver(listener);


	svg::SVGRectElement *rect = new svg::SVGRectElement(window->svgDocument);

	rect->setWidth(40);
	rect->setHeight(30);
	rect->setX(-20);
	rect->setY(-15);

	rect->setFill(0, 0, 1);
	rect->setStroke(0, 0, 0);
	rect->setStrokeWidth(2);
	rect->translate(40, 30);

	window->svgContent->appendChild(rect);
			
	hsm::ThreadLoop *t = new hsm::ThreadLoop(new Translator(window),  10);
	t->play();
	
	window->setVBLSync();
	window->show();

	hsm::eventloop::loop();

	delete t;
	delete window;
	
	return EXIT_SUCCESS;
}
